"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCookies = getCookies;
exports.setCookie = setCookie;
exports.deleteCookie = deleteCookie;
const property_accessors_1 = require("./property-accessors");
/**
 *
 * @this {RemoteDebugger}
 * @returns {Promise<import('@appium/types').StringRecord>}
 */
async function getCookies() {
    this.log.debug('Getting cookies');
    return await this.requireRpcClient().send('Page.getCookies', {
        appIdKey: (0, property_accessors_1.getAppIdKey)(this),
        pageIdKey: (0, property_accessors_1.getPageIdKey)(this),
    });
}
/**
 *
 * @this {RemoteDebugger}
 * @param {import('@appium/types').StringRecord} cookie
 * @returns {Promise<any>}
 */
async function setCookie(cookie) {
    this.log.debug('Setting cookie');
    return await this.requireRpcClient().send('Page.setCookie', {
        appIdKey: (0, property_accessors_1.getAppIdKey)(this),
        pageIdKey: (0, property_accessors_1.getPageIdKey)(this),
        cookie,
    });
}
/**
 *
 * @this {RemoteDebugger}
 * @param {string} cookieName
 * @param {string} url
 * @returns {Promise<any>}
 */
async function deleteCookie(cookieName, url) {
    this.log.debug(`Deleting cookie '${cookieName}' on '${url}'`);
    return await this.requireRpcClient().send('Page.deleteCookie', {
        appIdKey: (0, property_accessors_1.getAppIdKey)(this),
        pageIdKey: (0, property_accessors_1.getPageIdKey)(this),
        cookieName,
        url,
    });
}
/**
 * @typedef {import('../remote-debugger').RemoteDebugger} RemoteDebugger
 */
//# sourceMappingURL=cookies.js.map